/******************************************************************************/
/*!
\file		IJob.h
\project	Icarus Engine
\author		Jorge Antonio Jimenez

/brief		Interface that jobs can be inherited to be executed.
*/
/******************************************************************************/
#pragma once

#include <memory>
#include <string>

namespace IcarusEngine
{
	// The Status of any given job.
	enum class JobStatus
	{
	  Waiting,
	  Running,
	  Completed,
	  Canceled
	};

	// The Priority type the Job will have in the Queue
	enum class JobPriority
	{
		Low = 0,
		Medium,
		High,
		MaxPriority
	};

	// Forward declaration
	class IJob;

	// Typedef
	typedef std::shared_ptr<IJob> JobPtr;

	class IJob
	{
		public:
			IJob();
			virtual ~IJob();
			
			// Needs to be implemented specific to the type of Job.
			virtual void Execute()=0;

			// The Name of the Job for debugging.
			virtual std::string GetName()=0;

			JobPriority GetPriority();

			void PriorityUpgrade();

			void SetStatus(JobStatus status);

			JobStatus GetStatus();

			void AddAge(float dt);

			bool TimeOut();

			void TimeReset();

		protected:
			float m_Age;
			float m_MaxTime;
			JobStatus m_Status;
			JobPriority m_Priority;
	};
}