/******************************************************************************/
/*!
\file		MultiThread.cpp
\project	Icarus Engine
\author		Jorge Antonio Jimenez

\brief		Generates multiple Threads depending on the Cores on machine.
			To use it to paralize work.
*/
/******************************************************************************/
#pragma once

#include <list>
#include <string>
#include <vector>
#include <concurrent_queue.h>

#include <Engine\ISystem.h>
#include <Engine\MultiThread\Worker.h>
#include <Engine\MultiThread\IJob.h>

namespace IcarusEngine
{
	// Typedefs
	typedef std::list<JobPtr> JobList;

	// Forward Declaration
	void DoJob(JobPtr job);

	class MultiThread : public ISystem
	{
		public:
			MultiThread();
			virtual ~MultiThread();

			virtual void HandleMessage(Message* message);
			virtual void Initialize();
			virtual void Update(float dt);
			virtual std::string GetName();

			void AddingJob(JobPtr job);

			JobPtr GetNextJob();

			bool isWorking();

		private:
			// No Copy Constructor aloud.
			MultiThread(const MultiThread&);
			MultiThread& operator=(const MultiThread&);
			
			void Shutdown();
			void UpdatingJob(JobPtr job);
			bool JobIsWaiting(JobPtr job);

			std::vector<Worker> m_Workers;
			concurrency::concurrent_queue<JobPtr> m_PendingJobs[JobPriority::MaxPriority];
			concurrency::concurrent_queue<JobPtr> m_Tracker;
			bool m_Working;
			JobList m_JobTracker;
			int m_NotWorking;
	};

	extern MultiThread* MULTITHREAD;
}