/******************************************************************************/
/*!
\file		IJob.h
\project	Icarus Engine
\author		Jorge Antonio Jimenez

/brief		Interface that jobs can be inherited to be executed.
*/
/******************************************************************************/
#include <Engine\MultiThread\IJob.h>

namespace IcarusEngine
{
	/******************************************************************************/
	/*!
	  \brief
		Stores the Job with its data.
	*/
	/******************************************************************************/
	IJob::IJob()
		: m_Age(0.0f),
		m_MaxTime(10000.0f),
		m_Status(JobStatus::Waiting),
		m_Priority(JobPriority::Low)
	{
	}

	/******************************************************************************/
	/*!
	  \brief
		virtual destructor
	*/
	/******************************************************************************/
	IJob::~IJob()
	{
	}

	/******************************************************************************/
	/*!
	  \brief
		Returns the urgency of that job need to be executed.
	
	   \return
		JobPriority - Return what is the priority of the actual job.
	*/
	/******************************************************************************/
	JobPriority IJob::GetPriority()
	{
		return m_Priority;
	}

	/******************************************************************************/
	/*!
	  \brief
		It Upgrades the Job Priority, so it can be done earlier.
	
	   \return
		void
	*/
	/******************************************************************************/
	void IJob::PriorityUpgrade()
	{
		if(GetPriority() < JobPriority::High)
		{
			m_Priority = static_cast<JobPriority>(static_cast<unsigned int>(m_Priority) + 1);
		}
	}

	/******************************************************************************/
	/*!
	  \brief
		Returns the current status of the job.
	
	   \return
		JobStatus
	*/
	/******************************************************************************/
	JobStatus IJob::GetStatus()
	{
		return m_Status;
	}

	/******************************************************************************/
	/*!
	  \brief
		Set what is the status of the given job.
	
	   \return
		void
	*/
	/******************************************************************************/
	void IJob::SetStatus(JobStatus status)
	{
		m_Status = status;
	}

	/******************************************************************************/
	/*!
	  \brief
		Increase the amount of time that the job has been in the queue.
	
	   \return
		void
	*/
	/******************************************************************************/
	void IJob::AddAge(float dt)
	{
		m_Age += dt;
	}

	/******************************************************************************/
	/*!
	  \brief
		Says if the amount of time is reasonable for the job to be in the queue.
	
	   \return
		bool
	*/
	/******************************************************************************/
	bool IJob::TimeOut()
	{
		return m_Age >= m_MaxTime;
	}

	/******************************************************************************/
	/*!
	  \brief
		After the job has been upgraded the timer is reset.
	
	   \return
		void
	*/
	/******************************************************************************/
	void IJob::TimeReset()
	{
		m_Age = 0;
	}
}